const inboxifyFcState = {
    mappings: [],
    lists: {},
    formTitles: {},
    formFields: {},
};

document.addEventListener("DOMContentLoaded", () => {
    if (document.querySelector("#inboxify-fc-mappings-table")) {
        forceRefreshInboxifyLists();
        loadMappings();
        setupMappingForm();
    }
    if (document.querySelector("#inboxify-fc-save-settings")) {
        setupSettingsSave();
    }
});


function forceRefreshInboxifyLists() {
    const data = new URLSearchParams();
    data.append("action", "inboxify_fc_refresh_lists");
    data.append("security", inboxify_fc_ajax_object.nonce);

    fetch(ajaxurl, {
        method: "POST",
        body: data
    })
        .then(res => res.json())
        .then(response => {
            if (!response.success || !response.data.lists) return;
            const listMap = {};
            response.data.lists.forEach(list => {
                listMap[list.id] = list.list;
            });
            inboxifyFcState.lists = listMap;
            renderMappingsTable();
        })
        .catch(error => showSnackbar(inboxify_fc_i18n.errorLoadingLists.replace("%s", error), "error", 8000));
}


function loadMappings() {
    const data = new URLSearchParams();
    data.append("action", "inboxify_fc_get_mappings");
    data.append("security", inboxify_fc_ajax_object.nonce);

    fetch(ajaxurl, {
        method: "POST",
        body: data
    })
        .then(res => res.json())
        .then(response => {
            if (!response.success || !response.data.mappings) {
                displayNoMappingsFound();
                return;
            }

            inboxifyFcState.mappings = response.data.mappings;
            inboxifyFcState.lists = response.data.lists || {};
            inboxifyFcState.formTitles = response.data.form_titles || {};
            inboxifyFcState.formFields = {};

            renderMappingsTable();
            populateUnifiedFormSelect();
        })
        .catch(error => showSnackbar(inboxify_fc_i18n.errorFetchingMappings.replace("%s", error), "error", 8000));
}

function renderMappingsTable() {
    const tbody = document.querySelector("#inboxify-fc-mappings-table tbody");
    tbody.innerHTML = "";

    let mappingCount = 0;

    for (const [plugin, forms] of Object.entries(inboxifyFcState.mappings)) {
        for (const [formId, mapping] of Object.entries(forms)) {
            mappingCount++;
            const title = inboxifyFcState.formTitles[plugin]?.[formId] || formId;
            const label = plugin.toUpperCase() + " - " + title;
            const listLabel = inboxifyFcState.lists[mapping.list_id] || mapping.list_id;

            const tr = document.createElement("tr");
            tr.innerHTML = `
        <td><a href="#" class="edit-mapping" data-form-id="${formId}" data-plugin="${plugin}">${label}</a></td>
        <td>${listLabel}</td>
        <td><button class="delete-mapping button button-secondary" data-form-id="${formId}" data-plugin="${plugin}">${inboxify_fc_i18n.delete}</button></td>
      `;

            tbody.appendChild(tr);
        }
    }

    if (mappingCount === 0) {
        displayNoMappingsFound();
    }
}

function displayNoMappingsFound() {
    const tbody = document.querySelector("#inboxify-fc-mappings-table tbody");
    tbody.innerHTML = `<tr><td colspan='3'>${inboxify_fc_i18n.noMappingsFound}</td></tr>`;
}

document.addEventListener("click", (e) => {
    const editBtn = e.target.closest(".edit-mapping");
    const deleteBtn = e.target.closest(".delete-mapping");

    if (editBtn) {
        e.preventDefault();
        const plugin = editBtn.dataset.plugin;
        const formId = editBtn.dataset.formId;
        handleEditMapping(plugin, formId);
    }

    if (deleteBtn) {
        e.preventDefault();
        const plugin = deleteBtn.dataset.plugin;
        const formId = deleteBtn.dataset.formId;
        if (confirm(inboxify_fc_i18n.deleteMappingConfirm)) {
            deleteMapping(plugin, formId);
        }
    }
});

function handleEditMapping(plugin, formId) {
    resetMappingForm();
    document.querySelector("#inboxify-fc-mapping-form").style.display = "block";

    document.querySelector("#inboxify-fc-mapping-form-title").textContent = inboxify_fc_i18n.editMapping;

    const label = getFormLabel(plugin, formId);
    document.querySelector("#inboxify-fc-mapping-form-title").textContent = inboxify_fc_i18n.editMappingFor.replace("%s", label);

    document.querySelector("#inboxify-fc-form-selector-group").style.display = "none";

    populateUnifiedFormSelect(`${plugin}|${formId}`, true);
    loadFormFields(plugin, formId);
    loadInboxifyLists(plugin, formId);
}

function getFormLabel(plugin, formId) {
    const title = inboxifyFcState.formTitles?.[plugin]?.[formId] || formId;
    return `${plugin.toUpperCase()} - ${title}`;
}

function deleteMapping(plugin, formId) {
    const data = new URLSearchParams();
    data.append("action", "inboxify_fc_delete_mapping");
    data.append("plugin", plugin);
    data.append("form_id", formId);
    data.append("security", inboxify_fc_ajax_object.nonce);

    fetch(ajaxurl, {
        method: "POST",
        body: data
    })
        .then(res => res.json())
        .then(response => {
            if (response.success) {
                showSnackbar(inboxify_fc_i18n.mappingDeleted, "info");
                loadMappings();
            } else {
                showSnackbar(inboxify_fc_i18n.apiError + response.data.message, "error", 8000);
            }
        })
        .catch(error => showSnackbar(inboxify_fc_i18n.errorDeletingMapping.replace("%s", error), "error", 8000));;
}

function populateUnifiedFormSelect(selectedValue = null, disable = false) {
    const select = document.querySelector("#inboxify-fc-form-selector");
    if (!select) return;

    select.innerHTML = `<option value="">${inboxify_fc_i18n.selectForm}</option>`;

    const usedCombinations = new Set();
    for (const [plugin, forms] of Object.entries(inboxifyFcState.mappings)) {
        for (const [formId, mapping] of Object.entries(forms)) {
            const value = `${plugin}|${formId}`;
            usedCombinations.add(value);
        }
    }

    for (const [plugin, forms] of Object.entries(inboxifyFcState.formTitles)) {
        for (const [id, title] of Object.entries(forms)) {
            const value = `${plugin}|${id}`;
            const label = `${plugin.toUpperCase()} - ${title}`;
            const option = document.createElement("option");
            option.value = value;
            option.textContent = label;

            if (usedCombinations.has(value)) {
                option.disabled = true;
                option.textContent += ` ${inboxify_fc_i18n.alreadyMapped}`;
            }

            if (value === selectedValue) {
                option.selected = true;
            }

            select.appendChild(option);
        }
    }

    select.disabled = disable;
}


function loadFormFields(plugin, formId) {
    const cacheKey = `${plugin}_${formId}`;
    const cached = inboxifyFcState.formFields[cacheKey];
    if (cached) {
        const mapping = inboxifyFcState.mappings?.[plugin]?.[formId] || {};
        renderFieldMappingTable(cached, mapping.mapping || {}, mapping.list_id);
        return;
    }

    const data = new URLSearchParams();
    data.append("action", "inboxify_fc_get_form_fields");
    data.append("plugin", plugin);
    data.append("form_id", formId);
    data.append("security", inboxify_fc_ajax_object.nonce);

    fetch(ajaxurl, {
        method: "POST",
        body: data
    })
        .then(res => res.json())
        .then(response => {
            if (!response.success || !response.data.fields) {
                return;
            }
            inboxifyFcState.formFields[cacheKey] = response.data.fields;

            if (!inboxifyFcState.mappings[plugin]) {
                inboxifyFcState.mappings[plugin] = {};
            }
            inboxifyFcState.mappings[plugin][formId] = {
                mapping: response.data.mapping || {},
                list_id: response.data.list_id || ""
            };

            renderFieldMappingTable(response.data.fields, response.data.mapping || {}, response.data.list_id);
        })
        .catch(error => showSnackbar(inboxify_fc_i18n.errorFetchingFormFields.replace("%s", error), "error", 8000));
}

function renderFieldMappingTable(fields, currentMapping = {}, listId = null) {
    const tbody = document.querySelector("#inboxify-fc-field-mapping-table tbody");
    if (!tbody) return;

    tbody.innerHTML = "";

    fields.forEach(field => {
        const selected = currentMapping[field.name] || "__none__";

        const options = [`<option value=\"__none__\"${selected === "__none__" ? " selected" : ""}>${inboxify_fc_i18n.notMapped}</option>`];

        Object.keys(inboxifyFieldConfig).forEach(key => {
            const label = inboxifyFieldConfig[key]?.label?.nl || key;
            const selectedAttr = selected === key ? " selected" : "";
            options.push(`<option value=\"${key}\"${selectedAttr}>${label}</option>`);
        });

        const row = document.createElement("tr");
        row.innerHTML = `
      <td>${field.label}</td>
      <td>
        <select name=\"inboxify-fc-mapping[${field.name}]\" data-name=\"${field.name}\">${options.join("")}</select>
      </td>
    `;
        tbody.appendChild(row);
    });

    if (listId) {
        const listSelect = document.querySelector("#inboxify-fc-lists");
        if (listSelect) listSelect.value = listId;
    }
}

function resetMappingForm() {
    document.querySelector("#inboxify-fc-mapping-form-title").textContent = inboxify_fc_i18n.newMapping;
    document.querySelector("#inboxify-fc-save-mapping").textContent = inboxify_fc_i18n.save;
    document.querySelector("#inboxify-fc-form-selector").value = "";
    document.querySelector("#inboxify-fc-form-selector").disabled = false;
    document.querySelector("#inboxify-fc-form-selector-group").style.display = "block";
    document.querySelector("#inboxify-fc-lists").value = "";
    document.querySelector("#inboxify-fc-field-mapping-table tbody").innerHTML = "";
}

function loadInboxifyLists(plugin = null, formId = null, forceReload = false) {
    const select = document.querySelector("#inboxify-fc-lists");
    if (!select) return;

    if (!forceReload && Object.keys(inboxifyFcState.lists).length > 0) {
        renderListOptions(select, inboxifyFcState.lists, plugin, formId);
        return;
    }

    const data = new URLSearchParams();
    data.append("action", "inboxify_fc_fetch_lists");
    data.append("security", inboxify_fc_ajax_object.nonce);

    fetch(ajaxurl, {
        method: "POST",
        body: data
    })
        .then(res => res.json())
        .then(response => {
            if (!response.success || !response.data.lists) return;
            const listMap = {};
            response.data.lists.forEach(list => {
                listMap[list.id] = list.list;
            });
            inboxifyFcState.lists = listMap;
            renderListOptions(select, listMap, plugin, formId);
            updateSettingsListDisplay(listMap);
        })
        .catch(error => showSnackbar(inboxify_fc_i18n.errorLoadingLists.replace("%s", error), "error", 8000));
}

function renderListOptions(select, lists, plugin = null, formId = null) {
    select.innerHTML = `<option value="">${inboxify_fc_i18n.selectList}</option>`;

    for (const [id, name] of Object.entries(lists)) {
        const option = document.createElement("option");
        option.value = id;
        option.textContent = name;
        select.appendChild(option);
    }

    if (plugin && formId) {
        const mapping = inboxifyFcState.mappings?.[plugin]?.[formId];
        if (mapping && mapping.list_id) {
            select.value = mapping.list_id;
        }
    }
}

function setupSettingsSave() {
    const saveBtn = document.querySelector("#inboxify-fc-save-settings");
    if (!saveBtn) return;

    saveBtn.addEventListener("click", (e) => {
        e.preventDefault();
        const apiKey = document.querySelector("#inboxify-fc-api-key").value;
        const apiSecret = document.querySelector("#inboxify-fc-api-secret").value;

        const data = new URLSearchParams();
        data.append("action", "inboxify_fc_save_and_validate_api");
        data.append("api_key", apiKey);
        data.append("api_secret", apiSecret);
        data.append("security", inboxify_fc_ajax_object.nonce);

        fetch(ajaxurl, {
            method: "POST",
            body: data
        })
            .then(res => res.json())
            .then(response => {
                if (response.success) {
                    showSnackbar(inboxify_fc_i18n.apiConnected, "success");

                    const htmlContainer = document.querySelector("#inboxify-fc-lists");
                    if (htmlContainer && response.data.lists_html) {
                        htmlContainer.innerHTML = response.data.lists_html;
                    }
                } else {
                    showSnackbar(inboxify_fc_i18n.apiError + response.data, "error", 8000);
                }
            })
            .catch(error => showSnackbar(inboxify_fc_i18n.errorConnectingAPI.replace("%s", error), "error", 8000));;
    });
}

function setupMappingForm() {
    document.querySelector("#inboxify-fc-add-mapping")?.addEventListener("click", () => {
        resetMappingForm();
        document.querySelector("#inboxify-fc-mapping-form").style.display = "block";
        loadInboxifyLists();
    });

    document.querySelector("#inboxify-fc-cancel-mapping")?.addEventListener("click", () => {
        document.querySelector("#inboxify-fc-mapping-form").style.display = "none";
        resetMappingForm();
    });

    document.querySelector("#inboxify-fc-save-mapping")?.addEventListener("click", () => {
        const formSelectorValue = document.querySelector("#inboxify-fc-form-selector").value;
        if (!formSelectorValue) return showSnackbar(inboxify_fc_i18n.selectForm, "warning", 8000);
        const [plugin, formId] = formSelectorValue.split("|");

        const listId = document.querySelector("#inboxify-fc-lists").value;
        const mappings = [];
        document.querySelectorAll("#inboxify-fc-field-mapping-table select").forEach(select => {
            const name = select.dataset.name;
            const value = select.value;
            mappings.push([name, value]);
        });

        const data = new URLSearchParams();
        data.append("action", "inboxify_fc_save_mapping");
        data.append("plugin", plugin);
        data.append("form_id", formId);
        data.append("list_id", listId);
        data.append("mappings", JSON.stringify(mappings));
        data.append("security", inboxify_fc_ajax_object.nonce);

        fetch(ajaxurl, {
            method: "POST",
            body: data
        })
            .then(res => res.json())
            .then(response => {
                if (response.success) {
                    showSnackbar(inboxify_fc_i18n.mappingSaved, "success");

                    if (!inboxifyFcState.mappings[plugin]) inboxifyFcState.mappings[plugin] = {};
                    inboxifyFcState.mappings[plugin][formId] = {
                        mapping: Object.fromEntries(mappings),
                        list_id: listId
                    };

                    populateUnifiedFormSelect();

                    document.querySelector("#inboxify-fc-mapping-form").style.display = "none";

                    renderMappingsTable();
                } else {
                    showSnackbar(inboxify_fc_i18n.apiError + response.data.message, "warning", 8000);
                }
            })
            .catch(error => showSnackbar(inboxify_fc_i18n.errorSavingMapping.replace("%s", error), "error", 8000));;
    });

    document.querySelector("#inboxify-fc-form-selector")?.addEventListener("change", (e) => {
        const value = e.target.value;
        if (!value) return;

        const [plugin, formId] = value.split("|");
        loadFormFields(plugin, formId);
    });
}

function updateSettingsListDisplay(lists) {
    const container = document.querySelector("#inboxify-fc-lists");
    if (!container) return;

    if (!lists || Object.keys(lists).length === 0) {
        container.innerHTML = `<p>${inboxify_fc_i18n.noListsAvailable}</p>`;
        return;
    }

    let html = "<ul>";
    for (const [id, name] of Object.entries(lists)) {
        html += `<li>${name} (ID: ${id})</li>`;
    }
    html += "</ul>";
    container.innerHTML = html;
}

function showSnackbar(message, type = 'info', duration = 3000) {
    const snackbar = document.getElementById('inboxify-fc-snackbar');
    if (!snackbar) return;

    snackbar.className = 'inboxify-fc-snackbar';

    snackbar.classList.add(`inboxify-fc-snackbar-${type}`);

    snackbar.textContent = message;
    snackbar.classList.add('inboxify-fc-snackbar-show');

    setTimeout(() => {
        snackbar.classList.remove('inboxify-fc-snackbar-show');
        setTimeout(() => {
            snackbar.className = 'inboxify-fc-snackbar';
        }, 300);
    }, duration);
}