<?php
if ( ! defined( 'ABSPATH' ) ) exit;

add_action('wp_ajax_inboxify_fc_save_and_validate_api', function () {
    if (!current_user_can('manage_options')) {
        wp_send_json_error(__('You do not have permission to perform this action.', 'inboxify-form-connector'));
    }

    check_ajax_referer('inboxify_nonce', 'security');

    if (!isset($_POST['api_key'], $_POST['api_secret'])) {
        wp_send_json_error(__('API keys are missing.', 'inboxify-form-connector'));
    }

    $api_key = sanitize_text_field(wp_unslash($_POST['api_key']));
    $api_secret = sanitize_text_field(wp_unslash($_POST['api_secret']));

    if (empty($api_key) || empty($api_secret)) {
        wp_send_json_error(__('API keys are missing.', 'inboxify-form-connector'));
    }

    update_option('inboxify_fc_api_key', $api_key);
    update_option('inboxify_fc_api_secret', $api_secret);

    $lists = inboxify_fc_fetch_and_store_lists();

    if (is_wp_error($lists)) {
        wp_send_json_error($lists->get_error_message());
    }

    ob_start();
    echo '<ul>';
    foreach ($lists as $list) {
        echo '<li>' . esc_html($list['list']) . ' (ID: ' . esc_html($list['id']) . ')</li>';
    }
    echo '</ul>';
    $lists_html = ob_get_clean();

    wp_send_json_success(['lists_html' => $lists_html]);
});

function inboxify_fc_fetch_and_store_lists() {
    $api_key = get_option('inboxify_fc_api_key');
    $api_secret = get_option('inboxify_fc_api_secret');

    if (empty($api_key) || empty($api_secret)) {
        return new WP_Error('missing_keys', __('API keys are missing.', 'inboxify-form-connector'));
    }

    $response = wp_remote_get(INBOXIFY_FC_API_URL . '/lists', [
        'headers' => [
            'Authorization' => 'Basic ' . base64_encode($api_key . ':' . $api_secret),
        ],
    ]);

    if (is_wp_error($response)) {
        return new WP_Error('request_failed', __('Failed to connect to the Inboxify API.', 'inboxify-form-connector'));
    }

    $status_code = wp_remote_retrieve_response_code($response);
    $body = wp_remote_retrieve_body($response);
    $lists = json_decode($body, true);

    if ($status_code === 401 || $status_code === 403) {
        return new WP_Error('unauthorized', __('Invalid API credentials. Please check your key and secret.', 'inboxify-form-connector'));
    }

    if (!is_array($lists)) {
        return new WP_Error('invalid_response', __('Invalid response from the Inboxify API.', 'inboxify-form-connector'));
    }

    if (empty($lists)) {
        return new WP_Error('empty_lists', __('No lists available in your Inboxify account.', 'inboxify-form-connector'));
    }

    update_option('inboxify_fc_lists', $lists);
    return $lists;
}

add_action('wp_ajax_inboxify_fc_validate_connection', function () {
    check_ajax_referer('inboxify_nonce', 'security');

    $api_key = get_option('inboxify_fc_api_key');
    $api_secret = get_option('inboxify_fc_api_secret');

    if (empty($api_key) || empty($api_secret)) {
        wp_send_json_error(['message' => __('API keys are missing.', 'inboxify-form-connector')]);
    }

    $response = wp_remote_get(INBOXIFY_FC_API_URL . '/lists', [
        'headers' => [
            'Authorization' => 'Basic ' . base64_encode($api_key . ':' . $api_secret),
        ],
    ]);

    if (is_wp_error($response)) {
        wp_send_json_error(['message' => __('Error connecting to the Inboxify API.', 'inboxify-form-connector')]);
    }

    $lists = json_decode(wp_remote_retrieve_body($response), true);

    if (empty($lists) || !is_array($lists)) {
        wp_send_json_error(['message' => __('Invalid keys or no lists found.', 'inboxify-form-connector')]);
    }

    wp_send_json_success(['lists_count' => count($lists)]);
});

add_action('wp_ajax_inboxify_fc_fetch_lists', function () {
    check_ajax_referer('inboxify_nonce', 'security');

    $lists = get_option('inboxify_fc_lists', []);

    if (empty($lists)) {
        wp_send_json_error(['message' => __('No lists available.', 'inboxify-form-connector')]);
    }

    wp_send_json_success(['lists' => $lists]);
});

add_action('wp_ajax_inboxify_fc_refresh_lists', function () {
    if (!current_user_can('manage_options')) {
        wp_send_json_error(__('Unauthorized', 'inboxify-form-connector'));
    }

    check_ajax_referer('inboxify_nonce', 'security');

    $lists = inboxify_fc_fetch_and_store_lists();

    if (is_wp_error($lists)) {
        wp_send_json_error($lists->get_error_message());
    }

    wp_send_json_success(['lists' => $lists]);
});

function inboxify_fc_send_contact_to_list($list_id, array $payload) {
    $api_key = get_option('inboxify_fc_api_key');
    $api_secret = get_option('inboxify_fc_api_secret');

    if (empty($api_key) || empty($api_secret)) {
        return new WP_Error('missing_credentials', __('API key or secret is missing.', 'inboxify-form-connector'));
    }

    $response = wp_remote_post(INBOXIFY_FC_API_URL . "/contacts/{$list_id}?overwrite=true", [
        'headers' => [
            'Authorization' => 'Basic ' . base64_encode("$api_key:$api_secret"),
            'Content-Type'  => 'application/json',
        ],
        'body'    => json_encode($payload),
        'timeout' => 10,
    ]);

    if (is_wp_error($response)) {
        return $response;
    }

    $status = wp_remote_retrieve_response_code($response);
    $body = wp_remote_retrieve_body($response);

    if ($status >= 200 && $status < 300) {
        return true;
    }

    return new WP_Error(
        'api_error',
        sprintf(
            /* translators: %1$d = HTTP status code, %2$s = response body */
            esc_html__('Inboxify: API response (%1$d): %2$s', 'inboxify-form-connector'),
            $status,
            $body
        )
    );
}
