<?php
if ( ! defined( 'ABSPATH' ) ) exit;

function inboxify_fc_get_cf7_forms() {
    check_ajax_referer('inboxify_nonce', 'security');

    if (!function_exists('wpcf7')) {
        wp_send_json_error([
            "message" => __('Contact Form 7 is not installed or activated.', 'inboxify-form-connector')
        ]);
    }

    $forms = get_posts(['post_type' => 'wpcf7_contact_form', 'numberposts' => -1]);

    if (empty($forms)) {
        wp_send_json_error([
            "message" => __('No CF7 forms found.', 'inboxify-form-connector')
        ]);
    }

    $formatted_forms = [];
    foreach ($forms as $form) {
        $formatted_forms[] = ['id' => $form->ID, 'title' => $form->post_title];
    }

    wp_send_json_success(["forms" => $formatted_forms]);
}

function inboxify_fc_get_wpforms_forms() {
    check_ajax_referer('inboxify_nonce', 'security');

    if (!function_exists('wpforms')) {
        wp_send_json_error([
            "message" => __('WPForms is not active.', 'inboxify-form-connector')
        ]);
    }

    $forms = wpforms()->form->get('', ['orderby' => 'title', 'order' => 'ASC']);
    if (empty($forms)) {
        wp_send_json_success([]);
        return;
    }

    $result = [];
    foreach ($forms as $form) {
        $result[] = [
            'id'    => $form->ID,
            'title' => $form->post_title,
        ];
    }

    wp_send_json_success($result);
}

function inboxify_fc_get_form_fields() {
    check_ajax_referer('inboxify_nonce', 'security');

    $form_id = isset($_POST['form_id']) ? intval(wp_unslash($_POST['form_id'])) : 0;
    $plugin  = isset($_POST['plugin']) ? sanitize_text_field(wp_unslash($_POST['plugin'])) : 'cf7';

    $all_mappings   = get_option('inboxify_fc_mappings', []);
    $form_mapping   = $all_mappings[$plugin][$form_id] ?? [];
    $mapping_result = $form_mapping['mappings'] ?? [];
    $list_id        = $form_mapping['list_id'] ?? '';

    $form_fields = [];

    switch ($plugin) {
        case 'cf7':
            if (!function_exists('wpcf7_contact_form')) {
                wp_send_json_error([
                    "message" => __('Contact Form 7 is not active.', 'inboxify-form-connector')
                ]);
            }

            $contact_form = wpcf7_contact_form($form_id);
            if (!$contact_form) {
                wp_send_json_error([
                    "message" => __('Form not found.', 'inboxify-form-connector')
                ]);
            }

            foreach ($contact_form->scan_form_tags() as $tag) {
                if (!empty($tag->name)) {
                    $form_fields[] = [
                        'name'  => $tag->name,
                        'label' => $tag->name,
                    ];
                }
            }
            break;

        case 'wpforms':
            if (!function_exists('wpforms')) {
                wp_send_json_error([
                    "message" => __('WPForms is not active.', 'inboxify-form-connector')
                ]);
            }

            $form = wpforms()->form->get($form_id);
            if (!$form || empty($form->post_content)) {
                wp_send_json_error([
                    "message" => __('Form not found or empty.', 'inboxify-form-connector')
                ]);
            }

            $form_data = json_decode($form->post_content, true);
            if (empty($form_data['fields'])) {
                break;
            }

            foreach ($form_data['fields'] as $field_id => $field) {
                $form_fields[] = [
                    'name'  => 'field_' . $field_id,
                    'label' => $field['label'] ?? 'Field ' . $field_id,
                ];
            }
            break;

        case 'ninjaforms':
            if (!class_exists('Ninja_Forms')) {
                wp_send_json_error([
                    "message" => __('Ninja Forms is not active.', 'inboxify-form-connector')
                ]);
            }

            $form = Ninja_Forms()->form($form_id)->get();
            if (!$form || !method_exists($form, 'get_id')) {
                wp_send_json_error([
                    "message" => __('Ninja Forms: Form not found or invalid.', 'inboxify-form-connector')
                ]);
            }

            $fields = Ninja_Forms()->form($form_id)->get_fields();

            if (!is_array($fields) || empty($fields)) {
                wp_send_json_error([
                    "message" => __('Ninja Forms: No fields found.', 'inboxify-form-connector')
                ]);
            }

            foreach ($fields as $field) {
                if (!method_exists($field, 'get_setting')) continue;

                $type  = $field->get_setting('type') ?? '';
                $key   = $field->get_setting('key') ?? '';
                $label = $field->get_setting('label') ?? $key;

                if ($type === 'submit' || !$key) continue;

                $form_fields[] = [
                    'name'  => $key,
                    'label' => $label,
                ];
            }
            break;

        default:
            wp_send_json_error([
                /* translators: %s: plugin name */
                "message" => sprintf(__('Unknown plugin: %s', 'inboxify-form-connector'), $plugin)
            ]);
    }

    wp_send_json_success([
        'fields'   => $form_fields,
        'mapping'  => $mapping_result,
        'list_id'  => $list_id,
    ]);
}

function inboxify_fc_save_mapping() {
    if (!current_user_can('manage_options')) {
        wp_send_json_error([
            "message" => __('You do not have permission to perform this action.', 'inboxify-form-connector')
        ]);
    }

    check_ajax_referer('inboxify_nonce', 'security');

    $plugin  = isset($_POST['plugin']) ? sanitize_text_field(wp_unslash($_POST['plugin'])) : 'cf7';
    $form_id = isset($_POST['form_id']) ? sanitize_text_field(wp_unslash($_POST['form_id'])) : '';
    $list_id = isset($_POST['list_id']) ? sanitize_text_field(wp_unslash($_POST['list_id'])) : '';

    $mappings_raw = [];
    if (isset($_POST['mappings'])) {
        // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized -- Value is JSON decoded and fields are sanitized later
        $mappings_json = wp_unslash($_POST['mappings']);
        $mappings_raw = json_decode($mappings_json, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            wp_send_json_error([
                /* translators: %s = JSON error message */
                "message" => sprintf(__('JSON decode error: %s', 'inboxify-form-connector'), json_last_error_msg())
            ]);
        }
    }

    if (empty($form_id) || empty($list_id)) {
        wp_send_json_error([
            "message" => __('Form or list is missing.', 'inboxify-form-connector')
        ]);
    }

    $fixed_mapping = [];

    foreach ($mappings_raw as $pair) {
        if (!is_array($pair) || count($pair) !== 2) continue;

        [$form_field, $inboxify_field] = array_map('sanitize_text_field', $pair);

        if ($inboxify_field !== '__none__') {
            if (in_array($inboxify_field, $fixed_mapping)) {
                wp_send_json_error([
                    "message" => __('Each Inboxify field can only be mapped once.', 'inboxify-form-connector')
                ]);
            }
            $fixed_mapping[$form_field] = $inboxify_field;
        }
    }

    if (!in_array('email', $fixed_mapping)) {
        wp_send_json_error([
            "message" => __('You must map the "email" field.', 'inboxify-form-connector')
        ]);
    }

    $stored_mappings = get_option('inboxify_fc_mappings', []);
    if (!isset($stored_mappings[$plugin])) {
        $stored_mappings[$plugin] = [];
    }

    $stored_mappings[$plugin][$form_id] = [
        'list_id'  => $list_id,
        'mappings' => $fixed_mapping
    ];

    update_option('inboxify_fc_mappings', $stored_mappings);

    wp_send_json_success([
        "message" => __('Mapping saved successfully!', 'inboxify-form-connector')
    ]);
}

function inboxify_fc_get_mappings() {
    check_ajax_referer('inboxify_nonce', 'security');

    $mappings = maybe_unserialize(get_option('inboxify_fc_mappings', []));
    $lists = get_option('inboxify_fc_lists', []);

    $list_map = [];
    foreach ($lists as $list) {
        $list_map[$list['id']] = $list['list'];
    }

    $form_titles = [
        'cf7'        => [],
        'wpforms'    => [],
        'ninjaforms' => [],
    ];

    if (function_exists('wpcf7_contact_form')) {
        $cf7_forms = get_posts(['post_type' => 'wpcf7_contact_form', 'numberposts' => -1]);

        foreach ($cf7_forms as $form) {
            $form_titles['cf7'][$form->ID] = $form->post_title;
        }
    }

    if (post_type_exists('wpforms')) {
        $wpforms = get_posts(['post_type' => 'wpforms', 'numberposts' => -1]);

        foreach ($wpforms as $form) {
            $form_titles['wpforms'][$form->ID] = $form->post_title;
        }
    }

    if (class_exists('Ninja_Forms')) {
        $ninja_forms = Ninja_Forms()->form()->get_forms();

        if (is_array($ninja_forms) || is_object($ninja_forms)) {
            foreach ($ninja_forms as $form) {
                $form_titles['ninjaforms'][$form->get_id()] = $form->get_setting('title');
            }
        }
    }

    wp_send_json_success([
        'mappings' => $mappings,
        'lists' => $list_map,
        'form_titles' => $form_titles
    ]);
}

function inboxify_fc_delete_mapping() {
    check_ajax_referer('inboxify_nonce', 'security');

    $form_id = isset($_POST['form_id']) ? sanitize_text_field(wp_unslash($_POST['form_id'])) : '';
    $plugin  = isset($_POST['plugin']) ? sanitize_text_field(wp_unslash($_POST['plugin'])) : 'cf7';

    if (empty($form_id)) {
        wp_send_json_error([
            'message' => __('Form ID is missing.', 'inboxify-form-connector')
        ]);
    }

    $mappings = get_option('inboxify_fc_mappings', []);

    if (isset($mappings[$plugin][$form_id])) {
        unset($mappings[$plugin][$form_id]);

        if (empty($mappings[$plugin])) {
            unset($mappings[$plugin]);
        }

        update_option('inboxify_fc_mappings', $mappings);
        wp_send_json_success([
            'message' => __('Mapping deleted.', 'inboxify-form-connector')
        ]);
    } else {
        wp_send_json_error([
            'message' => __('Mapping not found.', 'inboxify-form-connector')
        ]);
    }
}

add_action('wp_ajax_inboxify_fc_get_cf7_forms', 'inboxify_fc_get_cf7_forms');
add_action('wp_ajax_inboxify_fc_get_wpforms_forms', 'inboxify_fc_get_wpforms_forms');
add_action('wp_ajax_inboxify_fc_get_form_fields', 'inboxify_fc_get_form_fields');
add_action('wp_ajax_inboxify_fc_delete_mapping', 'inboxify_fc_delete_mapping');
add_action('wp_ajax_inboxify_fc_save_mapping', 'inboxify_fc_save_mapping');
add_action('wp_ajax_inboxify_fc_get_mappings', 'inboxify_fc_get_mappings');